IMPLEMENTATION MODULE AmigaTime;

(* --------------------------------------------------------------
Author : Mark Olbert
      Bix:         chairmanmao
      Compuserve:  73557,1512
      Plink:       oks515

As always, all of this stuff is being put in the public domain, and is freely
redistributable, provided that it is not used in, or as a part of, a 
commercial package.  The source code was written in TDI's Modula 2 compiler
for the Amiga.

revised: 5/26/88 by Art Steinmetz
	Converted to Benchmark Modula 2
------------------------------------------------------------------ *)

FROM Time IMPORT Time,ValidTime,TimeToSerialSec,SerialSecToTime,SetTime,
                 SecsPerDay,TimeError;
FROM Date IMPORT Date,ValidDate,DateToSerialDay,SerialDayToDate,SetDate,
                 DateError;
FROM Strings IMPORT CopyString;                 
FROM IODevices IMPORT OpenDevice,CloseDevice;
FROM TimerDevice IMPORT TimerName,timeval,timerequest, timerequestPtr,
                        UnitVBlank,TRSetSysTime,TRGetSysTime;
FROM IODevices IMPORT IORequest,IORequestPtr,DoIO;
FROM Ports IMPORT MsgPortPtr;
FROM PortsUtil IMPORT CreatePort,DeletePort;
FROM IODevicesUtil IMPORT CreateExtIO,DeleteExtIO;
FROM SYSTEM IMPORT ADR, TSIZE;

TYPE
  TimerCondFlags = (port,request,device);
  
  TimerCondSet = SET OF TimerCondFlags;

PROCEDURE DateTimeToAmigaSecs(dval: Date; tval: Time; 
                             VAR secs: LONGINT): BOOLEAN; 
  VAR
    SDay: LONGINT;
    STime: LONGINT;
    BaseSDay: LONGINT;
    AmigaBirth: Date;
BEGIN
  secs:=0;
  IF ValidDate(dval) THEN
    IF ValidTime(tval) THEN
      DateToSerialDay(dval,SDay);
      TimeToSerialSec(tval,STime);
      IF NOT SetDate(AmigaBirthYear,AmigaBirthMonth,AmigaBirthDay,AmigaBirth) THEN
        AmigaTimeError:="Illegal Amiga birthdate parameters!!";
        RETURN FALSE;
      ELSE
        DateToSerialDay(AmigaBirth,BaseSDay);
        secs:=SecsPerDay*(SDay-BaseSDay)+STime;
        RETURN TRUE;
      END;
    ELSE
      CopyString(AmigaTimeError,TimeError);
      RETURN FALSE;
    END;
  ELSE
    CopyString(AmigaTimeError,DateError);
    RETURN FALSE;
  END;
END DateTimeToAmigaSecs;
  
PROCEDURE AmigaSecsToDateTime(VAR dval: Date; VAR tval: Time; 
                              secs: LONGINT): BOOLEAN; 
  VAR                             
    SDay: LONGINT;
    STime: LONGINT;
    BaseSDay: LONGINT;
    AmigaBirth: Date;
BEGIN
  WITH dval DO
    Year:=0;
    Month:=0;
    Day:=0;
  END;
  WITH tval DO
    Hour:=0;
    Minute:=0;
    Second:=0;
  END;
  SDay:=secs DIV SecsPerDay;
  IF secs < 0D THEN 
    SDay:=-SDay;
    DEC(SDay);
  END;
  STime:=secs-SecsPerDay*SDay;
  WITH AmigaBirth DO
    Year:=1978;
    Month:=1;
    Day:=1;
  END;
  DateToSerialDay(AmigaBirth,BaseSDay);
  SDay:=SDay+BaseSDay;  
  SerialSecToTime(tval,STime);
  SerialDayToDate(dval,SDay);
  IF ValidDate(dval) THEN
    IF ValidTime(tval) THEN
      RETURN TRUE;
    ELSE
      CopyString(AmigaTimeError,TimeError);
      RETURN FALSE;
    END;
  ELSE
    CopyString(AmigaTimeError,DateError);
  END;
END AmigaSecsToDateTime;
  
PROCEDURE EndSysTimer(statset: TimerCondSet; tmreq: timerequestPtr;
                      tport: MsgPortPtr);
BEGIN  
  IF device IN statset THEN CloseDevice(tmreq) END;
  IF request IN statset THEN 
          DeleteExtIO(IORequestPtr(tmreq)); 
  END;
  IF port IN statset THEN DeletePort(tport^) END;
END EndSysTimer;

PROCEDURE InitSysTimer(VAR tport: MsgPortPtr; 
                       VAR tmreq: timerequestPtr): BOOLEAN;  
  VAR                       
    Status: TimerCondSet;
BEGIN
  Status:=TimerCondSet{};
  tport:=CreatePort(ADR(""),0);
  IF tport=NIL THEN
    AmigaTimeError:="Couldn't create port for handling timer device requests.";
  ELSE    
    INCL(Status,port);
    tmreq:=timerequestPtr(CreateExtIO(tport^,TSIZE(timerequest)));
    IF tmreq=NIL THEN
      AmigaTimeError:="Couldn't create timer device request structure.";
    ELSE
      INCL(Status,request);
      IF OpenDevice(ADR(TimerName),UnitVBlank,tmreq,0D)<>0D THEN
        AmigaTimeError:="Couldn't open timer device.";
      ELSE
        INCL(Status,device);
      END;
    END;
  END;
  IF Status # TimerCondSet{port,request,device} THEN
    EndSysTimer(Status,tmreq,tport);
    RETURN FALSE;
  ELSE
    RETURN TRUE;
  END;
END InitSysTimer;

PROCEDURE SetSysTime(newdate: Date; newtime: Time): BOOLEAN;
  VAR
    TPort: MsgPortPtr;
    TimerReq: timerequestPtr;
    NewSecs: LONGINT;
    RetVal: BOOLEAN;
BEGIN
  RetVal:=FALSE;
  IF DateTimeToAmigaSecs(newdate,newtime,NewSecs) THEN
    IF InitSysTimer(TPort,TimerReq) THEN
      TimerReq^.trtime.tvsecs:=LONGCARD(NewSecs);
      TimerReq^.trtime.tvmicro:=0;
      TimerReq^.trnode.ioCommand:=TRSetSysTime;
      IF DoIO(ADR(TimerReq^.trnode)) <> 0D THEN
        AmigaTimeError:="Error on DoIO setting system time";
      ELSE
        RetVal:=TRUE;
      END;
      EndSysTimer(TimerCondSet{port,request,device},TimerReq,TPort);
    END;
  END;
  RETURN RetVal;
END SetSysTime;

PROCEDURE GetSysTime(VAR sysdate: Date; VAR systime: Time): BOOLEAN;
  VAR
    TPort: MsgPortPtr;
    TimerReq: timerequestPtr;
    SysSecs: LONGINT;
    RetVal: BOOLEAN;
BEGIN
  RetVal:=FALSE;
  IF InitSysTimer(TPort,TimerReq) THEN
    TimerReq^.trnode.ioCommand:=TRGetSysTime;
    IF DoIO(ADR(TimerReq^.trnode))<> 0D THEN
      AmigaTimeError:="Error on DoIO getting system time";
    ELSE
      SysSecs:=LONGINT(TimerReq^.trtime.tvsecs);
      IF TimerReq^.trtime.tvmicro >= 500000D THEN INC(SysSecs) END;
      RetVal:=AmigaSecsToDateTime(sysdate,systime,SysSecs);
    END;
    EndSysTimer(TimerCondSet{port,request,device},TimerReq,TPort);
  END;
  RETURN RetVal;
END GetSysTime;

END AmigaTime.
