MODULE AtomClock;

(* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	Version: 1.0  9/24/88 by Art Steinmetz

	Public Domain. Please give credit where due though.

	Usage:  ATOMCLOCK [<flag>/<value>]
	Optional Arguments:

z - Time zone (i.e. EST, PDT). Use "o" flag below if zone is not
                 range EDT..PST.
o - Hours offset from GMT.
b - BAUD rate.
n - Phone number
p - Modem dial prefix
r - Modem reset string
c - How many calls to make before giving up.
t - Try to read X lines before giving up.
s - Modem dial suffix
y - Modem 'busy' message.
w - Modem 'no answer' message.

Typing ATOMCLOCK without any arguments is equivalent to typing
    ATOMCLOCK z/EST o/-5 b/1200 n/1-202-653-0351 p/ATDT r/ATZ^M c/3 t/20
         s/^M b/BUSY w/NO CARRIER

 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ *)
(* $R+ $V+ *)

FROM SYSTEM	   	IMPORT ADR, BYTE, ADDRESS;
FROM AmigaDOSProcess    IMPORT Delay;

FROM SimpleConsole      IMPORT CreateConsole, ClearInput, DeleteConsole,
			       GetAgain, GetCh, GetStr, PutCh, PutStr, PutBuf,
			       TestCh;

FROM Intuition	 IMPORT WindowPtr, CloseWindow, IDCMPFlags,
		   IDCMPFlagsSet, WindowFlags, WindowFlagsSet,
		   IntuiMessagePtr;
FROM Ports	 IMPORT GetMsg, WaitPort, ReplyMsg, MsgPortPtr,
		   FindPort, MessagePtr;

FROM Tasks	 IMPORT SignalSet, Wait;

FROM Serial      IMPORT OpenSer, CloseSer, SerRead, SerWrite, QuerySer,
		    ReadPort, QueueSerRead, SynchronizeIO;

FROM StringUtils IMPORT String;

FROM Strings     IMPORT ConcatString, StringLength, 
                    LocateSubString, CopyString;

FROM Time IMPORT Time; 
(*
FROM Date IMPORT Date;
*)
FROM AtomUtils IMPORT 	AtomParamRec, AtomDefaults, ComputeZone,
			AccumulateLine, ParseUTC, ProcessMoment,
			OpenAtomWindow, ProgName;
						
FROM GetAtomParams IMPORT GetParams;

CONST
	MonitorProgress = TRUE;   (* Debugging Flag *)

        LF = 12C;
        CR = 15C;

VAR
  CreatedConsole,
  OpenedSer,
  OnlyHelp     : BOOLEAN;
  SerialSigBit : CARDINAL;
  MySignalSet  : SignalSet;
  Params       : AtomParamRec;
  wp           : WindowPtr;

(* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ *)
PROCEDURE SerWriteLn;

VAR
  ch : CHAR;
  
BEGIN
    ch := CR;
    SerWrite(ch,1);
    ch := LF;
    SerWrite(ch,1);
END SerWriteLn;

(* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ *)
PROCEDURE ListenUp(VAR MySignalSet : SignalSet);


BEGIN
   SerialSigBit := CARDINAL(ReadPort^.mpSigBit); (* modem wakes us up *);
   MySignalSet := SignalSet{};          (* Intialize *)
   INCL(MySignalSet,SerialSigBit);
   INCL(MySignalSet,CARDINAL(wp^.UserPort^.mpSigBit)); (* Intuition too *)
END ListenUp;

PROCEDURE SendInterpreted(str : ARRAY OF CHAR);
TYPE
    ByteSet = SET OF [0..7];
VAR
    i : CARDINAL;
    ch : CHAR;
BEGIN
    i := 0;
    WHILE (i <= HIGH(str)) AND (str[i]<>0C) DO
	IF str[i] = '^' THEN
	    INC(i);
	    ch := CHAR(ByteSet(str[i]) - ByteSet{6});
	    SerWrite(ch,1);
	ELSIF str[i] = '\\' THEN
	    INC(i);
	    IF str[i] = 'w' THEN
		Delay(50)
	    END
	ELSE
	    SerWrite(str[i],1);
	END;
	INC(i)
    END;
END SendInterpreted;
   
(* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ *)
PROCEDURE CallTheFeds(P : AtomParamRec; VAR Success : BOOLEAN);
   
VAR
  finish   : BOOLEAN;
  class    : IDCMPFlagsSet;
  sig      : SignalSet;
  MsgPtr : IntuiMessagePtr;
  GMT      : Time;
(*  TheDate  : Date; *)
  i,
  tries,
  code     : CARDINAL;
  TempChar : CHAR;
  TempStr  : String;

BEGIN
    finish := FALSE;
    Success := FALSE;
    tries := 0;
    PutStr(wp^,ADR("Dialing Naval Observatory"));
    PutCh(wp^,LF);
    WITH Params DO
	IF Prefix[0] <> 0C THEN SendInterpreted(Prefix) END;
	SendInterpreted(PhoneNumber);
	IF Suffix[0] <> 0C THEN SendInterpreted(Suffix) END;
    END;
    QueueSerRead(TempChar);
    REPEAT
     (* Wait for intuition or serial message *)
	sig := Wait(MySignalSet);

	IF SerialSigBit IN sig THEN
	    SynchronizeIO;
	    IF MonitorProgress THEN 
		PutCh(wp^,TempChar)
	    END;
	    IF AccumulateLine(TempStr,TempChar) THEN   (* Get A Line *)
		INC(tries);
		IF ParseUTC(TempStr, 
			GMT (*, TheDate *) ) THEN  (* Is is A Time? *)
		    IF ProcessMoment(wp,GMT (*, TheDate *) ,
		    			ComputeZone(Params)) THEN
			finish := TRUE;
			Success := TRUE;
		    END
		ELSE
		    IF LocateSubString(TempStr,Params.Busy,0,
				StringLength(TempStr)) > -1 THEN 
			PutStr(wp^,ADR("Line is Busy"));
			PutCh(wp^,LF);
			finish := TRUE;
		    END;
		    IF LocateSubString(TempStr,Params.NoAnswer,0,
				StringLength(TempStr)) > -1 THEN 
			PutStr(wp^,ADR("No pickup on other end."));
			PutCh(wp^,LF);
			finish := TRUE;
		    END;
		END; 
		TempStr[0] := 0C;  (* Clear String *) 
	    END;
	    IF tries >= P.MaxTries THEN finish := TRUE END;    
            QueueSerRead(TempChar);
	ELSE
            QueueSerRead(TempChar);
        END;

	MsgPtr := GetMsg (wp^.UserPort^);  (* check Intuition *)
	IF MsgPtr <> NIL THEN 
	    class := MsgPtr^.Class;
	    code := MsgPtr^.Code;
 	    ReplyMsg (MessagePtr(MsgPtr));
	    IF (class = IDCMPFlagsSet {Closewindow}) THEN
		finish := TRUE;
		Success := TRUE; (* not really but we want out *)
                PutStr(wp^,ADR("Close gadget hit.  Bye."));
	    END;
	END;
    UNTIL finish;
END CallTheFeds;



(* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ *)
PROCEDURE Atomic;

VAR
  Calls : CARDINAL;
  NavObsPhone : String;
  Success  : BOOLEAN;

BEGIN
   Calls := 1;

   wp := OpenAtomWindow();

   (* open serial port at 8 data bits, 1 stop bit *)
   OpenSer(Params.BAUD,BYTE(1),BYTE(8),CHAR(011H),CHAR(013H),0C,0C);
   OpenedSer := TRUE;

   ListenUp(MySignalSet);  (* Listen to the window and serial port *)

   WITH Params DO
	IF Reset[0] <> 0C THEN
	    SendInterpreted(Reset);
	    Delay(100);
	END;
   END;

   IF CreateConsole(wp^) THEN (* Here's the beef *)
     CreatedConsole := TRUE;
     REPEAT
       CallTheFeds(Params, Success);
       INC(Calls);
     UNTIL (Calls > Params.MaxCalls) OR Success;   
     IF (Calls > Params.MaxCalls) AND NOT Success THEN
       PutStr(wp^,ADR("Giving up trying to call."));
     END; 
     DeleteConsole(wp^);
     CreatedConsole := FALSE;
   END;

   Delay(100);
END Atomic;


PROCEDURE Cleanup;
BEGIN
    IF CreatedConsole THEN
	DeleteConsole(wp^);  (* should be redundant *)
    END;
    IF OpenedSer THEN
	CloseSer
    END;
    IF wp <> NIL THEN
	CloseWindow (wp^);
    END;
END Cleanup;
(* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ *)

BEGIN (* Main *)

   wp := NIL;
   CreatedConsole := FALSE;
   OpenedSer := FALSE;
   Params := AtomDefaults;
   GetParams(Params, OnlyHelp);
   IF NOT OnlyHelp THEN
      Atomic;
   END;
   Cleanup;
END AtomClock.
