IMPLEMENTATION MODULE AtomUtils;

(* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        For use with AtomClock.mod
	Version: 1.0  9/24/88 by Art Steinmetz

	Public Domain. Please give credit where due though.

 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ *)

FROM SYSTEM IMPORT ADR;
FROM Intuition	 IMPORT IDCMPFlags, IDCMPFlagsSet, NewWindow, 
			WindowFlagsSet, WindowFlags, OpenWindow,
			WBenchScreen, WindowPtr;
FROM Time IMPORT Time, TimePhase, TimeToString, SetTime, ValidTime,
			SecsPerDay,
			TimeToSerialSec, SerialSecToTime; 


FROM Date IMPORT Date (*, SerialDayToDate - NOT IMPLEMENTED *);

FROM AmigaTime IMPORT SetSysTime, GetSysTime;

FROM Strings IMPORT ConcatString, CopyString, 
			StringLength, ExtractSubString;

FROM StringUtils IMPORT String, CardToString, StringToCard, InitParse,
			ParseString, SUError, SeparatorType;

FROM SimpleConsole      IMPORT PutStr, PutCh;

FROM SYSTEM IMPORT BYTE;

FROM Conversions IMPORT ConvStringToNumber;

CONST
   LF = 12C;
   CR = 15C;
   WindowTitle = "AtomClock V 1.0";

VAR
   BaseDate : LONGINT;

PROCEDURE OpenAtomWindow() : WindowPtr;

VAR
  win : NewWindow;

  
BEGIN
  WITH win DO
    LeftEdge	:= 100;
    TopEdge	:= 100;
    Width	:= 320;
    Height	:= 75;
    DetailPen	:= BYTE(-1);
    BlockPen	:= BYTE(-1);
    Title	:= ADR(WindowTitle);
    IDCMPFlags	:= IDCMPFlagsSet { Closewindow, VanillaKey};
    Flags	:= WindowFlagsSet{ Activate, WindowClose, 
    			WindowDrag, WindowDepth };
    FirstGadget := NIL;
    Type	:= WBenchScreen;
  END;
  
  RETURN OpenWindow(win);
END OpenAtomWindow;  

(* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ *)
PROCEDURE AccumulateLine(VAR LineStr : ARRAY OF CHAR; ch : CHAR) : BOOLEAN;

VAR
 len : CARDINAL;

BEGIN
  IF (ch = LF) OR (ch = CR) THEN
     RETURN TRUE
  ELSE
     len := StringLength(LineStr);
     IF len > 254 THEN RETURN TRUE END;  (* force end of line *)
     LineStr[len] := ch;
     LineStr[len + 1] := 0C;
     RETURN FALSE;
  END;
END AccumulateLine;

(* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ *)
PROCEDURE ComputeZone(p : AtomParamRec ) : INTEGER;

BEGIN
  CASE p.TimeZone OF
	EST..PST : RETURN -(INTEGER(ORD(p.TimeZone))) + p.TimeOffset |
	EDT..PDT : RETURN -(INTEGER(ORD(p.TimeZone))) + 5 + p.TimeOffset |
	GMT,UTC  : RETURN 0 + p.TimeOffset |
    ELSE
      RETURN p.TimeOffset;
    END;
END ComputeZone;	  	
(* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ *)

(* $D- *)
PROCEDURE ParseUTC(UTCStr : ARRAY OF CHAR;
                   VAR TheTime : Time (* ;
		   VAR TheDate : Date -NOT IMPLEMENTED- *) ) : BOOLEAN;
(* TRUE equals success *)

CONST
  spc = " ";

TYPE
  UTCRec = RECORD
     DaysIntoCentury, DaysIntoYear, Now : String;
  END;
 
VAR
  temp,
  delim      : String;
  Yr         : LONGINT;
  Hr,Min,Sec : CARDINAL;
  UTC        : UTCRec;
  
BEGIN
  BaseDate := 0;
  CopyString(delim,spc);
  IF InitParse(delim) THEN
    IF NOT ParseString(UTCStr,UTC.DaysIntoCentury) THEN 
       RETURN FALSE 
    ELSE 
       IF NOT ConvStringToNumber(UTC.DaysIntoCentury,Yr,FALSE,10) THEN 
         RETURN FALSE 
       END; 
  (*     SerialDayToDate(TheDate,BaseDate + Yr); *)
    END;
    IF NOT ParseString(UTCStr,UTC.DaysIntoYear) THEN 
       RETURN FALSE 
    END;       
    IF NOT ParseString(UTCStr,UTC.Now) THEN 
       RETURN FALSE 
    ELSE 
       ExtractSubString(temp,UTC.Now,0,2);
       IF NOT StringToCard(temp,Hr) THEN RETURN FALSE END;
       ExtractSubString(temp,UTC.Now,2,2);
       IF NOT StringToCard(temp,Min) THEN RETURN FALSE END;
       ExtractSubString(temp,UTC.Now,4,2);
       IF NOT StringToCard(temp,Sec) THEN RETURN FALSE END;
       IF NOT SetTime(Hr,Min,Sec,mil,TheTime) THEN RETURN FALSE END;
    END;
  ELSE RETURN FALSE
  END;
  RETURN TRUE
END ParseUTC;
(* $D+ *)

(* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ *)
PROCEDURE ProcessMoment( wp : WindowPtr; (* console window pointer *)
			 Moment : Time; 
			 (* Day : Date; - NOT IMPLEMENTED *) 
			 TZ : INTEGER) : BOOLEAN;
VAR
  sysdate : Date;
  systime : Time;
  SerMoment : LONGINT;
  TempStr : String;

BEGIN
    PutStr(wp^,ADR("GMT is "));
    TimeToString(Moment,TempStr);      
    PutStr(wp^,ADR(TempStr));
    PutCh(wp^,LF);
    TimeToSerialSec(Moment,SerMoment);
   (* Convert GMT to local time *)
    SerMoment := SerMoment + LONGINT(TZ * 3600);
    IF SerMoment < 0D THEN 
	SerMoment := SerMoment + SecsPerDay
    END;
    SerialSecToTime(Moment,SerMoment);

    IF ValidTime(Moment) THEN 
	PutStr(wp^,ADR("  Local Time is "));
	TimeToString(Moment,TempStr);      
	PutStr(wp^,ADR(TempStr));
	PutCh(wp^,LF);
	IF GetSysTime(sysdate,systime) THEN
	    PutStr(wp^,ADR("System time was "));
	    TimeToString(systime,TempStr);      
	    PutStr(wp^,ADR(TempStr));
	    PutCh(wp^,LF);
	    IF SetSysTime(sysdate,Moment) THEN
		PutStr(wp^,ADR("Changing to "));
		TimeToString(Moment,TempStr);      
		PutStr(wp^,ADR(TempStr));
		PutCh(wp^,LF);
		RETURN TRUE;
	    ELSE
		PutStr(wp^,ADR("COULD NOT SET SYSTIME"));
		RETURN FALSE;
	    END;
	ELSE 
	    PutStr(wp^,ADR("Invalid Time after converting to local!"));
	    RETURN FALSE
	END ;
	PutCh(wp^,LF);
    END;
    RETURN FALSE
END ProcessMoment;

BEGIN
   WITH AtomDefaults DO
      TimeOffset  := -5;   (* EST = GMT - 5 *)
      PhoneNumber := '1-202-653-0351';
      BAUD        := 1200; (* don't know if others are available *)
      TimeZone    := EST;    (* Eastern Standard Time *)
      MaxCalls    := 3;
      MaxTries    := 20;
      Reset       := 'ATZ^M';
      Prefix      := 'ATDT';
      Suffix	  := '^M';
      Busy	  := 'BUSY';
      NoAnswer	  := 'NO CARRIER';
  END;
END AtomUtils.
