IMPLEMENTATION MODULE Date;
(* --------------------------------------------------------------
Author : Mark Olbert
      Bix:         chairmanmao
      Compuserve:  73557,1512
      Plink:       oks515

As always, all of this stuff is being put in the public domain, and is freely
redistributable, provided that it is not used in, or as a part of, a 
commercial package.  The source code was written in TDI's Modula 2 compiler
for the Amiga.

revised: 5/26/88 by Art Steinmetz
	Converted to Benchmark Modula 2
------------------------------------------------------------------ *)

FROM StringUtils IMPORT SeparatorType,ConvertSeparator,CardToString,
			String,
                        LowerCaseString,InitParse,ParseString;
FROM Strings IMPORT StringLength,CopyString,ConcatString;          
FROM Conversions IMPORT ConvStringToNumber;

TYPE
  
  DateDisplay = RECORD
      Sequence: DateDisplaySequence;
      MonthStyle: MonthDisplayStyle;
      YearStyle: YearDisplayStyle;
      FirstSeparator,SecondSeparator: ARRAY[0..2] OF CHAR;
    END;
  
VAR    
  DateFormat: DateDisplay;
  
PROCEDURE SetDateFormat(seq: DateDisplaySequence; mstyle: MonthDisplayStyle;
                        ystyle: YearDisplayStyle; sep1,sep2: SeparatorType);
BEGIN
  WITH DateFormat DO
    Sequence:=seq;
    MonthStyle:=mstyle;
    YearStyle:=ystyle;
    ConvertSeparator(sep1,FirstSeparator);
    ConvertSeparator(sep2,SecondSeparator);
  END;
END SetDateFormat;
  
PROCEDURE MonthToName(mnum: CARDINAL; VAR mname: ARRAY OF CHAR);
BEGIN
  CASE mnum OF
    1: CopyString(mname,"January"); |
    2: CopyString(mname,"February"); |
    3: CopyString(mname,"March"); |
    4: CopyString(mname,"April"); |
    5: CopyString(mname,"May"); |
    6: CopyString(mname,"June"); |
    7: CopyString(mname,"July"); |
    8: CopyString(mname,"August"); |
    9: CopyString(mname,"September"); |
    10: CopyString(mname,"October"); |
    11: CopyString(mname,"November"); |
    12: CopyString(mname,"December");
  ELSE
    CopyString(mname,"*** undefined ***");
  END;
END MonthToName;

PROCEDURE NameToMonth(mname: ARRAY OF CHAR; VAR mnum: CARDINAL): BOOLEAN;
BEGIN
  LowerCaseString(mname);
  IF StringLength(mname)>3 THEN mname[3]:=00C END;
  CASE mname[0] OF
    'a':
      IF mname[1]='p' THEN
        mnum:=4;
      ELSE
        mnum:=8;
      END;|
    'd': mnum:=12;|
    'f': mnum:=2;|
    'j':
      IF mname[1]='a' THEN
        mnum:=1;
      ELSE
        IF mname[2]='n' THEN
          mnum:=6;
        ELSE
          mnum:=7;
        END;
      END;|
    'm': mnum:=5;|
    'n': mnum:=11;|
    'o': mnum:=10;|
    's': mnum:=9;|
  ELSE
    mnum:=0;
  END;
  IF mnum=0 THEN
    RETURN FALSE;
  ELSE
    RETURN TRUE;
  END;
END NameToMonth;

PROCEDURE DateToString(dateval: Date; VAR out: ARRAY OF CHAR);
  VAR                       
    MonthText: String;
    DayText: String;
    YearText: String;
    TempStg: String;
    Century: CARDINAL;
    YearNum: CARDINAL;
    Width: CARDINAL;
BEGIN
  WITH dateval DO
    IF DateFormat.MonthStyle=numeric THEN
      CardToString(Month,"0",2,MonthText);
    ELSE
      MonthToName(Month,MonthText);
      IF DateFormat.MonthStyle=shortname THEN MonthText[3]:=0C END;
    END;
    CardToString(Day,"0",2,DayText);
    IF DateFormat.YearStyle=short THEN
      Century:=Year DIV 100;
      YearNum:=Year-100*Century;
      CardToString(YearNum,"0",2,YearText);
    ELSE
      YearNum:=Year;
      CardToString(YearNum,"0",4,YearText);
    END;

    out[0]:=0C;
    CASE DateFormat.Sequence OF
      MDY:
        CopyString(out,MonthText);
        ConcatString(out,DateFormat.FirstSeparator);
	ConcatString(out,DayText);
        ConcatString(out,DateFormat.SecondSeparator);
	ConcatString(out,YearText);|
      DMY:
        CopyString(out,DayText);
        ConcatString(out,DateFormat.FirstSeparator);
        ConcatString(out,MonthText);
        ConcatString(out,DateFormat.SecondSeparator);
        ConcatString(out,YearText);
    END;
  END;
END DateToString;

PROCEDURE ValidDate(testval: Date): BOOLEAN;
  VAR
    ErrorMessage: String;
BEGIN
  ErrorMessage:="";
  WITH testval DO
    IF Month<1 THEN ErrorMessage:="Month less than 1!" END;
    IF Month>12 THEN ErrorMessage:="Month greater than 12!" END;
    IF Day<1 THEN ErrorMessage:="Day less than 1!" END;
    IF Day>DaysInMonth(Year,Month) THEN 
      ErrorMessage:="Day greater than days in month!";
    END;
    IF Year<1 THEN ErrorMessage:="Year less than 1!" END;
  END;
  IF ErrorMessage[0]<>00C THEN
    DateError:=ErrorMessage;
    RETURN FALSE;
  ELSE
    RETURN TRUE;
  END;
END ValidDate;

PROCEDURE DaysInMonth(yearval: CARDINAL; 
                      monthval: CARDINAL): CARDINAL;
BEGIN                      
  CASE monthval OF
    1,3,5,7,8,10,12: 
      RETURN 31; |
    4,6,9,11: 
      RETURN 30; |
    2:
      IF IsLeapYear(yearval) THEN
        RETURN 29;
      ELSE
        RETURN 28;
      END;
  ELSE
    RETURN 0;
  END;
END DaysInMonth;      

PROCEDURE IsLeapYear(yearval: CARDINAL): BOOLEAN;
  VAR
    YearMod4,YearMod400: CARDINAL;
BEGIN
  YearMod4:=yearval-4*(yearval DIV 4);
  YearMod400:=yearval-400*(yearval DIV 400);
  IF YearMod4=0 THEN
    IF YearMod400<>0 THEN
      RETURN TRUE;
    ELSE
      RETURN FALSE;
    END;
  ELSE
    RETURN FALSE;
  END;
END IsLeapYear;

PROCEDURE DaysUntilMonth(yearval: CARDINAL; monthval: CARDINAL): CARDINAL;
  VAR
    CumulativeDays: CARDINAL;
    MonthCount: CARDINAL;
BEGIN
  CumulativeDays:=0;
  MonthCount:=1;
  FOR MonthCount:=1 TO monthval-1 DO
    INC(CumulativeDays,DaysInMonth(yearval,MonthCount));
  END;
  RETURN CumulativeDays;  
END DaysUntilMonth;

PROCEDURE DaysInYear(yearval: CARDINAL): CARDINAL;
BEGIN
  IF IsLeapYear(yearval) THEN
    RETURN 366;
  ELSE
    RETURN 365;
  END;
END DaysInYear;

PROCEDURE DateToSerialDay(dateval: Date; VAR sernum: LONGINT);
  VAR
    Ahead: BOOLEAN;
    MatchYear: CARDINAL;
BEGIN
  IF BaseYear<=dateval.Year THEN
    Ahead:=TRUE;
    MatchYear:=BaseYear;
  ELSE
    Ahead:=FALSE;
    MatchYear:=BaseYear-1;
  END;
  sernum:=0;
  WHILE MatchYear<>dateval.Year DO
    IF Ahead THEN
      INC(sernum,DaysInYear(MatchYear));
      INC(MatchYear);
    ELSE
      DEC(sernum,DaysInYear(MatchYear));
      DEC(MatchYear);
    END;
  END;
  IF Ahead THEN
    INC(sernum,DaysUntilMonth(dateval.Year,dateval.Month)+dateval.Day);    
  ELSE
    DEC(sernum,DaysInYear(MatchYear)-DaysUntilMonth(dateval.Year,dateval.Month)-dateval.Day);
  END;
END DateToSerialDay;

PROCEDURE SerialDayToDate(VAR dateval: Date; sernum: LONGINT);
  VAR
    CurYear: CARDINAL;
    CurMonth: CARDINAL;
    CurDay: CARDINAL;
    Forward: BOOLEAN;
    DaysInCurPeriod: LONGINT;
BEGIN
    IF sernum > 0D THEN
      Forward:=TRUE;
      CurYear:=BaseYear;
      CurMonth:=1;
    ELSE
      Forward:=FALSE;
      CurYear:=BaseYear-1;
      CurMonth:=12;
      sernum:=ABS(sernum)+1D;
    END;
    DaysInCurPeriod:=LONGINT(DaysInYear(CurYear));
    WHILE sernum>DaysInCurPeriod DO
      DEC(sernum,DaysInCurPeriod);
      IF Forward THEN
        INC(CurYear);
      ELSE
        DEC(CurYear);
      END;
      DaysInCurPeriod:=LONGINT(DaysInYear(CurYear));
    END;
    DaysInCurPeriod:=LONGINT(DaysInMonth(CurYear,CurMonth));
    WHILE sernum>DaysInCurPeriod DO
      DEC(sernum,DaysInCurPeriod);
      IF Forward THEN
        INC(CurMonth);
      ELSE
        DEC(CurMonth);
      END;
      DaysInCurPeriod:=LONGINT(DaysInMonth(CurYear,CurMonth));
    END;
    WITH dateval DO
      Year:=CurYear;
      Month:=CurMonth;
      IF Forward THEN
        Day:= CARDINAL(sernum);
      ELSE
        Day:= CARDINAL(DaysInCurPeriod-sernum+1D);
      END;
    END;
END SerialDayToDate;

PROCEDURE DaysBetweenDates(date1,date2: Date): LONGINT;
  VAR
    serial1,serial2: LONGINT;
BEGIN
  DateToSerialDay(date1,serial1);
  DateToSerialDay(date2,serial2);
  RETURN serial2-serial1;
END DaysBetweenDates;

PROCEDURE NewDate(VAR dateval: Date; numdays: LONGINT);
  VAR
    serval: LONGINT;
BEGIN
  DateToSerialDay(dateval,serval);
  SerialDayToDate(dateval,serval+numdays);
END NewDate;

PROCEDURE SetDate(year,month,day: CARDINAL; VAR dateval: Date): BOOLEAN;
BEGIN
  WITH dateval DO
    Year:=year;
    Month:=month;
    Day:=day;
  END;
  IF NOT ValidDate(dateval) THEN
    WITH dateval DO
      Year:=BaseYear;
      Month:=1;
      Day:=1;
    END;
    RETURN FALSE;
  ELSE
    RETURN TRUE;
  END;
END SetDate;

PROCEDURE DateFormatToString(VAR fmtstg: ARRAY OF CHAR);
  VAR
    TempStg: String;
    MFormat,DFormat,YFormat: String;
BEGIN
  CASE DateFormat.MonthStyle OF
    numeric:   MFormat:="MM";|
    shortname: MFormat:="<3 Letter Month Name>";|
    longname:  MFormat:="<Full Month Name>";
  END;
  CASE DateFormat.YearStyle OF
    short: YFormat:="YY";|
    long:  YFormat:="YYYY";
  END;
  DFormat:="DD";
  CASE DateFormat.Sequence OF
    MDY:
      ConcatString(fmtstg,MFormat);
      ConcatString(fmtstg,DateFormat.FirstSeparator);
      ConcatString(fmtstg,DFormat);|
    DMY:
      ConcatString(fmtstg,DFormat);
      ConcatString(fmtstg,DateFormat.FirstSeparator);
      ConcatString(fmtstg,MFormat);|
  END;
  ConcatString(fmtstg,DateFormat.SecondSeparator);
  ConcatString(fmtstg,YFormat);
END DateFormatToString;

PROCEDURE StringToDate(instg: ARRAY OF CHAR; VAR newdate: Date): BOOLEAN;
  VAR
    Token: ARRAY[0..9] OF CHAR;
    ConvResult: BOOLEAN;
    NumArgs: CARDINAL;
    NewYear,NewMonth,NewDay: CARDINAL;
    CurArg: LONGCARD;
    BrkLoop,InputOK: BOOLEAN;
    Sep,TempStg: String;
    MType: CHAR;
    ArgTypes: ARRAY[0..2] OF CHAR;
    ArgNames: ARRAY[0..3] OF CHAR;
BEGIN
  CopyString(Sep,DateFormat.FirstSeparator);
  ConcatString(Sep,DateFormat.SecondSeparator);
  ConcatString(Sep," ");
  IF DateFormat.MonthStyle=numeric THEN
    MType:="n";
  ELSE
    MType:="s";
  END;
  IF DateFormat.Sequence=MDY THEN
    ArgTypes[0]:=MType;
    ArgTypes[1]:="n";
    ArgTypes[2]:="n";
    ArgNames:="mdy";
  ELSE
    ArgTypes[0]:="n";
    ArgTypes[1]:=MType;
    ArgTypes[2]:="n";
    ArgNames:="dmy";
  END;
  IF NOT InitParse(Sep) THEN
    DateError:="StringToDate:  InitParse failed!!";
    InputOK:=FALSE;
  ELSE
    NumArgs:=0;
    BrkLoop:=FALSE;
    InputOK:=TRUE;
    WHILE (NOT BrkLoop) AND (InputOK) DO
      IF NOT ParseString(instg,Token) THEN
        InputOK:=FALSE;
        BrkLoop:=TRUE;
        DateError:="StringToDate:  Parsing of input string failed!!";
      ELSE
        IF ArgTypes[NumArgs]="n" THEN
          IF ConvStringToNumber(Token,CurArg,TRUE,10) THEN
            CASE ArgNames[NumArgs] OF
              'm': NewMonth:=CARDINAL(CurArg);|
              'd': NewDay:=CARDINAL(CurArg);|
              'y': NewYear:=CARDINAL(CurArg);
            END;
          ELSE
            DateError:="StringToDate:  character data in a numeric field!!";
          END;
        ELSE
          IF NOT NameToMonth(Token,NewMonth) THEN
            InputOK:=FALSE;
            DateError:="StringToDate:  invalid month name!!";
            BrkLoop:=FALSE;
          END;
        END;
        IF StringLength(instg)=0 THEN
          IF NumArgs<2 THEN 
            InputOK:=FALSE;
            DateError:="StringToDate:  not enough fields specified!!";
          END;
          BrkLoop:=TRUE;
        ELSIF NumArgs=2 THEN BrkLoop:=TRUE END;
      END;
      IF NOT BrkLoop THEN INC(NumArgs) END;
    END;
    IF InputOK THEN 
      IF DateFormat.YearStyle=short THEN INC(NewYear,1900) END;
      IF NOT SetDate(NewYear,NewMonth,NewDay,newdate) THEN
        InputOK:=FALSE;
        DateError:="StringToDate:  Illegal month, day or year values!!";
      END;
    END;
    RETURN InputOK;
  END;
END StringToDate;

BEGIN
  SetDateFormat(MDY,numeric,short,slash,slash);
END Date.
