IMPLEMENTATION MODULE GetAtomParams;

(* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        For use with AtomClock.mod
	Version: 1.0  9/24/88 by Art Steinmetz

	Public Domain. Please give credit where due though.

 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ *)
			

FROM AtomUtils   IMPORT AtomDefaults, AtomParamRec, TimeZones, 
			ProgName;

FROM AmigaDOS	IMPORT Close, Open, ModeNewFile;
FROM Conversions IMPORT ConvStringToNumber;

FROM Libraries IMPORT OpenLibrary, CloseLibrary;

FROM Strings     IMPORT CompareString, DeleteSubString, 
			CopyString, Relation;
FROM StringUtils IMPORT String, UpperCaseString;
FROM SYSTEM      IMPORT ADR, ADDRESS, LONGWORD;
FROM System IMPORT HALTX, argc, argv, WBenchMsg, 
		StdInput,StdOutput;
FROM TermInOut IMPORT WriteString, WriteLn, WriteHex, Write;
FROM Termination IMPORT AddTerminator;
FROM Workbench IMPORT IconBase, IconName, 
			FindToolType, GetDiskObject,
			DiskObjectPtr;
TYPE
    ByteSet = SET OF [0..7];

(* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ *)
PROCEDURE Help;
BEGIN

WriteString("Usage:  ATOMCLOCK [<flag>/<value> ...]\n");
WriteString("Optional Arguments:\n");
WriteString('   z - Time zone (i.e. EST, PDT). Use "o" flag\n');
WriteString("    if zone is not in range EDT..PST.\n");
WriteString("   o - Hours offset from GMT.\n");
WriteString("   b - BAUD rate.\n");
WriteString("   n - Phone number\n");
WriteString("   p - Modem dial prefix\n");
WriteString("   s - Modem dial suffix\n");
WriteString("   r - Modem reset string\n");
WriteString("   y - Modem 'busy' message\n");
WriteString("   w - Modem 'no answer' message\n");
WriteString("   c - How many calls to make before giving up.\n");
WriteString("   t - Try to read X lines before giving up.\n");
WriteString("^M signifies a carraige return\n");
WriteString(
   "\nTyping ATOMCLOCK without any arguments is equivalent to typing\n"); 
WriteString( 
"ATOMCLOCK z/EST o/-5 b/1200 n/1-202-653-0351 p/ATDT r/ATZ^M c/3 t/20\n"); 
WriteString( 
"     s/^M b/BUSY w/NO ANSWER\n"); 

END Help;

(* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ *)
PROCEDURE GetZone(VAR p : AtomParamRec; instr : ARRAY OF CHAR);
  BEGIN
  UpperCaseString(instr);
  IF CompareString(instr,'EST') = equal THEN p.TimeZone :=  EST END;
  IF CompareString(instr,'CST') = equal THEN p.TimeZone :=  CST END;
  IF CompareString(instr,'MST') = equal THEN p.TimeZone :=  MST END;
  IF CompareString(instr,'PST') = equal THEN p.TimeZone :=  PST END;
  IF CompareString(instr,'EDT') = equal THEN p.TimeZone :=  EDT END;
  IF CompareString(instr,'CDT') = equal THEN p.TimeZone :=  CDT END;
  IF CompareString(instr,'MDT') = equal THEN p.TimeZone :=  MDT END;
  IF CompareString(instr,'PDT') = equal THEN p.TimeZone :=  PDT END;
  IF CompareString(instr,'GMT') = equal THEN p.TimeZone :=  GMT END;
  IF CompareString(instr,'UTC') = equal THEN p.TimeZone :=  UTC END;
END GetZone;


(* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ *)
PROCEDURE GetInteger(	instr : ARRAY OF CHAR;  
			FailValue : INTEGER) : INTEGER;

VAR
  tempnum : LONGWORD;

BEGIN

  IF ConvStringToNumber(instr, tempnum, TRUE, 10) THEN
     RETURN INTEGER(tempnum)
  ELSE
     RETURN FailValue   
  END;

END GetInteger;


PROCEDURE Translate(VAR str : ARRAY OF CHAR);
(* Translate occurence of ^x to Control-X. *)
(* For example ^M is translated to carriage-return *)
VAR
    i,j : CARDINAL;
BEGIN
    j := 0;
    i := 0;
    WHILE j <= HIGH(str) DO
	IF (str[j] = '^') AND (j < HIGH(str)) THEN
	    INC(j);
	    str[i] :=  CHAR(ByteSet(str[j]) - ByteSet{6});
	ELSE
	    str[i] := str[j]
	END;
	INC(j);
	INC(i)
    END;
    str[i] := 0C;
i := 0;
WHILE (i <= HIGH(str)) AND (str[i] <> 0C) DO
    WriteHex(ORD(str[i]),2); Write(' ');
    INC(i)
END;
END Translate;

(* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*)
PROCEDURE GetWBParams(	VAR p : AtomParamRec;
			AppName : ADDRESS );
VAR
  id 		: DiskObjectPtr;
  toolarray	: ADDRESS;
  tool		: POINTER TO String;

BEGIN
(* Get rid of the extra window Benchmark opens *)
  Close(StdOutput);
  StdOutput := Open(ADR("NIL:"), ModeNewFile);
  StdInput := StdOutput;

  IconBase := OpenLibrary(ADR(IconName),0);
  IF IconBase = NIL THEN HALTX END;
  id := GetDiskObject(AppName);
  toolarray := id^.doToolTypes;

  tool := FindToolType(toolarray, ADR("TimeZone"));   
  IF NOT (tool = NIL) THEN GetZone(p,tool^); END;

  tool := FindToolType(toolarray, ADR("PhoneNumber"));   
  IF NOT (tool = NIL) THEN CopyString(p.PhoneNumber,tool^); END;
  
  tool := FindToolType(toolarray, ADR("TimeOffset"));   
  IF NOT (tool = NIL) THEN 
     p.TimeOffset := GetInteger(tool^,p.TimeOffset);
  END;
  
  tool := FindToolType(toolarray, ADR("BAUD"));
  IF NOT (tool = NIL) THEN p.BAUD := GetInteger(tool^,p.BAUD); END;
  
  tool := FindToolType(toolarray, ADR("MaxCalls"));   
  IF NOT (tool = NIL) THEN 
     p.MaxCalls := GetInteger(tool^,p.MaxCalls);
  END;
  tool := FindToolType(toolarray, ADR("MaxTries"));   
  IF NOT (tool = NIL) THEN 
     p.MaxTries := GetInteger(tool^,p.MaxTries);
  END;
  tool := FindToolType(toolarray, ADR("Prefix"));   
  IF NOT (tool = NIL) THEN CopyString(p.Prefix,tool^) END;

  tool := FindToolType(toolarray, ADR("Reset"));   
  IF NOT (tool = NIL) THEN CopyString(p.Reset,tool^) END;

  tool := FindToolType(toolarray, ADR("Suffx"));   
  IF NOT (tool = NIL) THEN CopyString(p.Suffix,tool^) END;

  tool := FindToolType(toolarray, ADR("NoAnswer"));   
  IF NOT (tool = NIL) THEN CopyString(p.NoAnswer,tool^) END;

  tool := FindToolType(toolarray, ADR("Busy"));   
  IF NOT (tool = NIL) THEN CopyString(p.Busy,tool^) END;

END GetWBParams;

(* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ *)
PROCEDURE GetCLIParams(VAR p : AtomParamRec; VAR JustHelp : BOOLEAN);
(* template for CL args is <flag char>/<value> *)

VAR
  arg    : String;
  flagch : CHAR;
  i      : CARDINAL;
   
BEGIN
  JustHelp := FALSE;
  i := 1;
  IF argc > 1 THEN  (* started from CLI AND some CL args *)
    REPEAT
      CopyString(arg,argv^[i]^);
      flagch := arg[0];
      DeleteSubString(arg,0,2);

      CASE CAP(flagch) OF
        'B' : p.BAUD := GetInteger(arg,p.BAUD)
	|
        'N' : CopyString(p.PhoneNumber,arg)
	|
        'Z' : GetZone(p, arg)
	|
        'O' : p.TimeOffset := GetInteger(arg,p.TimeOffset)
	|
        'C' : p.MaxCalls := GetInteger(arg,p.MaxCalls)
	|
        'T' : p.MaxTries := GetInteger(arg,p.MaxTries)
	|
        'P' : CopyString(p.Prefix,arg)
	|
        'R' : CopyString(p.Reset,arg);
	|
        'S' : CopyString(p.Suffix,arg)
	|
        'Y' : CopyString(p.Busy,arg)
	|
        'W' : CopyString(p.NoAnswer,arg)
	|
	'?' : Help; JustHelp := TRUE
	ELSE
	    WriteString('\nUnknown parameter.\n')
        END;
     INC(i);
    UNTIL  i = argc;
  END;
END GetCLIParams;


(*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*)
PROCEDURE GetParams(VAR p : AtomParamRec; VAR JustHelp : BOOLEAN);
BEGIN
   IF argc > 0 THEN 
      GetCLIParams(p, JustHelp)
   ELSE 
      GetWBParams(p, ADR(ProgName));
      JustHelp := FALSE;
   END;
END GetParams;


(*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ *)
PROCEDURE CloseUp;
BEGIN
  IF IconBase <> NIL THEN CloseLibrary(IconBase^) END;
END CloseUp;
(* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*)
BEGIN
  IconBase := NIL;
  AddTerminator(CloseUp);
END GetAtomParams.
