IMPLEMENTATION MODULE StringUtils;
(* --------------------------------------------------------------
Author : Mark Olbert
      Bix:         chairmanmao
      Compuserve:  73557,1512
      Plink:       oks515

As always, all of this stuff is being put in the public domain, and is freely
redistributable, provided that it is not used in, or as a part of, a 
commercial package.  The source code was written in TDI's Modula 2 compiler
for the Amiga.

revised: 5/26/88 by Art Steinmetz
	Converted to Benchmark Modula 2
	added StringToCard routine
------------------------------------------------------------------ *)

FROM Conversions IMPORT ConvNumberToString, ConvStringToNumber;
FROM Strings IMPORT CopyString,StringLength,LocateChar,
		ExtractSubString,ConcatString;
FROM InOut IMPORT Read,EOL;


VAR
  ParseDelims: ARRAY[0..MaxDelims] OF CHAR;
  NumDelims: CARDINAL;
  SpcIsDelim: BOOLEAN;

PROCEDURE ConvertSeparator(sep: SeparatorType; VAR sepstring: ARRAY OF CHAR);
  VAR
    Count: CARDINAL;
BEGIN
  IF HIGH(sepstring)<2 THEN
    FOR Count:=0 TO HIGH(sepstring)-1 DO
      sepstring[Count]:='*';
    END;
    sepstring[Count+1]:=0C;
  ELSE
    CASE sep OF
      space: 
        sepstring[0]:=' '; 
        sepstring[1]:=0C; |
      slash: 
        sepstring[0]:='/'; 
        sepstring[1]:=0C; |
      dash: 
        sepstring[0]:='-'; 
        sepstring[1]:=0C; |
      colon: 
        sepstring[0]:=':'; 
        sepstring[1]:=0C; |
      comma: 
        sepstring[0]:=',';
        sepstring[1]:=' ';
        sepstring[2]:=0C; |      
      nothing: 
        sepstring[0]:=0C;
    END;
  END;
END ConvertSeparator;

PROCEDURE TrimLeadingBlanks(VAR stg: ARRAY OF CHAR);
  VAR
    CharCount: CARDINAL;
    StgLen: CARDINAL;
    FirstNonBlank: CARDINAL;
    NonBlankFound: BOOLEAN;
BEGIN
  StgLen:=StringLength(stg);
  NonBlankFound:=FALSE;
  FOR CharCount:=0 TO StgLen DO
    IF NOT NonBlankFound THEN
      IF stg[CharCount]<>' ' THEN
        NonBlankFound:=TRUE;
        FirstNonBlank:=CharCount;
      END;
    END;
    IF NonBlankFound THEN stg[CharCount-FirstNonBlank]:=stg[CharCount] END;
  END;
END TrimLeadingBlanks;

PROCEDURE TrimTrailingBlanks(VAR stg: ARRAY OF CHAR);
  VAR
    CharCount: INTEGER;
BEGIN
  CharCount:=StringLength(stg);
  WHILE (stg[CharCount]=' ') AND (CharCount>=0) DO
    DEC(CharCount);
  END;
  stg[CharCount+1]:=0C;
END TrimTrailingBlanks;

PROCEDURE UpperCaseString(VAR stg: ARRAY OF CHAR);
  VAR
    temp   : CARDINAL;
    StgLen: CARDINAL;
    CharCount: CARDINAL;
BEGIN
  StgLen:=StringLength(stg);
  FOR CharCount:=0 TO StgLen DO
    IF (stg[CharCount]>='a') AND (stg[CharCount]<='z') THEN 
      temp := CARDINAL(stg[CharCount]);
      DEC(temp,32);
      stg[CharCount] := CHAR(temp);
    END;
  END;
END UpperCaseString;

PROCEDURE LowerCaseString(VAR stg: ARRAY OF CHAR);
  VAR
    temp  : CARDINAL;
    StgLen: CARDINAL;
    CharCount: CARDINAL;
BEGIN
  StgLen:=StringLength(stg);
  FOR CharCount:=0 TO StgLen DO
    IF (stg[CharCount]>='A') AND (stg[CharCount]<='Z') THEN
      temp := CARDINAL(stg[CharCount]);
      INC(temp,32);
      stg[CharCount] := CHAR(temp);
    END;
  END;
END LowerCaseString;

PROCEDURE PadFront(VAR stg: ARRAY OF CHAR; padstg: ARRAY OF CHAR; 
                   times: CARDINAL);
  VAR
    Skeleton : String;
    RepCount: CARDINAL;
BEGIN
  Skeleton[0]:=00C;
  FOR RepCount:=1 TO times DO
    ConcatString(Skeleton,padstg);
  END;
  ConcatString(Skeleton,stg);
  CopyString(stg,Skeleton);
END PadFront;

PROCEDURE PadBack(VAR stg: ARRAY OF CHAR; padstg: ARRAY OF CHAR; 
                  times: CARDINAL);
  VAR
    Skeleton,TempStg: String;
    RepCount: CARDINAL;
BEGIN
  Skeleton[0]:=00C;
  FOR RepCount:=1 TO times DO
    ConcatString(Skeleton,padstg);
  END;
  ConcatString(stg,Skeleton);
END PadBack;
         
PROCEDURE CardToString(num: CARDINAL; padchar: CHAR; mindigits: CARDINAL;
                       VAR out: ARRAY OF CHAR); 
  VAR
    ConvNumber: String;
    PadStg: ARRAY[0..1] OF CHAR;
BEGIN
  ConvNumberToString (ConvNumber,LONGCARD(num),FALSE,10,mindigits,padchar);
  CopyString(out,ConvNumber);
END CardToString;

PROCEDURE StringToCard(stgnum : ARRAY OF CHAR; 
                       VAR num : CARDINAL) : BOOLEAN;
VAR
  longnum : LONGCARD;

BEGIN
   IF ConvStringToNumber(stgnum,longnum,FALSE,10) THEN
        IF longnum < 65536D THEN (* check for cardinal overflow *)
	   num := CARDINAL(longnum);
	   RETURN TRUE
        ELSE RETURN FALSE END;
   ELSE RETURN FALSE END;	   
END StringToCard;	

PROCEDURE ReadLine(VAR line: ARRAY OF CHAR);
  VAR
    LineSize,CharNum: CARDINAL;
    CurChar: CHAR;
BEGIN
  LineSize:=HIGH(line);
  CharNum:=0;
  REPEAT
    Read(CurChar);
    IF (CurChar=EOL) OR (CharNum=LineSize) THEN CurChar:=00C END;
    line[CharNum]:=CurChar;
    INC(CharNum);
  UNTIL CurChar=00C;
END ReadLine;

PROCEDURE InitParse(delims: ARRAY OF CHAR): BOOLEAN;
  VAR
    CharCount: CARDINAL;
BEGIN
  NumDelims:=0;
  CharCount:=0;
  SpcIsDelim:=FALSE;
  WHILE (NumDelims<MaxDelims) AND (delims[CharCount]<>00C) DO
    IF delims[CharCount]=' ' THEN
      SpcIsDelim:=TRUE;
    ELSE
      ParseDelims[NumDelims]:=delims[CharCount];
      INC(NumDelims);
    END;
    INC(CharCount);
  END;
  IF (NumDelims=MaxDelims) AND (delims[CharCount]<>00C) THEN
    ParseDelims[0]:=00C;
    RETURN FALSE;
  ELSE
    ParseDelims[NumDelims]:=00C;
    RETURN TRUE;
  END;
END InitParse;

PROCEDURE ParseString(VAR src: ARRAY OF CHAR; 
                      VAR token: ARRAY OF CHAR): BOOLEAN;
  VAR
    BrkScan,OnToken,GotToken: BOOLEAN;
    CharCount,TokCount,Dummy: CARDINAL;
    SrcLength: CARDINAL;
    NewSrc: ARRAY[0..80] OF CHAR;
BEGIN
  GotToken:=FALSE;
  IF (ParseDelims[0]<>00C) OR (SpcIsDelim = TRUE) THEN
    SrcLength:=StringLength(src);
    BrkScan:=FALSE;
    OnToken:=FALSE;
    CharCount:=0;
    TokCount:=0;
    WHILE NOT BrkScan DO
      IF ((src[CharCount]=' ') AND (SpcIsDelim) AND (OnToken)) OR 
         (LocateChar(ParseDelims,src[CharCount],0,
	        StringLength(ParseDelims)) > -1) THEN
        BrkScan:=TRUE;
        GotToken:=TRUE;
        token[TokCount]:=00C;
      ELSIF (src[CharCount]>=' ') AND (src[CharCount]<='~') THEN
        IF src[CharCount]<>' ' THEN
          OnToken:=TRUE;
        END;
        IF OnToken THEN
          token[TokCount]:=src[CharCount];
          INC(TokCount);
        END;
      ELSIF src[CharCount]<>' ' THEN
        SUError:="ParseString:  Illegal char or delim not found!!";
        BrkScan:=TRUE;
      END;
      INC(CharCount);
      IF src[CharCount]=00C THEN
        BrkScan:=TRUE;
        GotToken:=TRUE;
        token[TokCount]:=00C;
      END;
    END;
  ELSE
    SUError:="ParseString:  No delimiters specified for parsing!!";
  END;
  IF GotToken THEN
    IF CharCount>=SrcLength THEN
      src[0]:=00C;
    ELSE
      ExtractSubString(NewSrc,src,CharCount,SrcLength-CharCount);
      CopyString(src,NewSrc);
    END;
  END;
  RETURN GotToken;
END ParseString;

END StringUtils.
