IMPLEMENTATION MODULE Time;
(* --------------------------------------------------------------
Author : Mark Olbert
      Bix:         chairmanmao
      Compuserve:  73557,1512
      Plink:       oks515

As always, all of this stuff is being put in the public domain, and is freely
redistributable, provided that it is not used in, or as a part of, a 
commercial package.  The source code was written in TDI's Modula 2 compiler
for the Amiga.

revised: 5/26/88 by Art Steinmetz
	Converted to Benchmark Modula 2
------------------------------------------------------------------ *)

FROM StringUtils IMPORT SeparatorType,ConvertSeparator,CardToString,
			String,
                        LowerCaseString,InitParse,ParseString;
FROM Strings IMPORT StringLength,CopyString,ConcatString;          
FROM Conversions IMPORT ConvStringToNumber;

TYPE
  
  TimeDisplay = RECORD
      Style: TimeStyles;
      Separator: ARRAY[0..2] OF CHAR;
      AMPM: TimePhase;
    END;
    
VAR    
  TimeFormat: TimeDisplay;
  
PROCEDURE SetTimeFormat(style: TimeStyles; amp: TimePhase; sep: SeparatorType);
BEGIN
  WITH TimeFormat DO
    Style:=style;
    ConvertSeparator(sep,Separator);
    AMPM:=amp;
  END;
END SetTimeFormat;

PROCEDURE TimeToString(timeval: Time; VAR out: ARRAY OF CHAR);
  VAR
    Converted: String;
    HourVal: CARDINAL;
    AmPmString: String;
    TempStg: String;
BEGIN
  WITH timeval DO
    IF TimeFormat.AMPM<>mil THEN
      CASE Hour OF
        0:
          HourVal:=12;
          AmPmString:=" a.m.";|
        12:
          HourVal:=12;
          AmPmString:=" p.m.";|
        1..11:
          HourVal:=Hour; 
          AmPmString:=" a.m.";|
        13..23:
          HourVal:=Hour-12;
          AmPmString:=" p.m.";
      ELSE
        HourVal:=0;
        AmPmString:="*** error ***";
      END;
    ELSE
      HourVal:=Hour;
      AmPmString:="";
    END;
    IF HourVal > 9 THEN 
      CardToString(HourVal,"0",2,out)
    ELSE
      CardToString(HourVal,"0",1,out)
    END;
    ConcatString(out,TimeFormat.Separator);
    CardToString(Minute,"0",2,Converted);
    ConcatString(out, Converted);
    IF TimeFormat.Style=HHMMSS THEN
      ConcatString(out,TimeFormat.Separator);
      CardToString(Second,"0",2,Converted);
      ConcatString(out,Converted);
    END;
    ConcatString(out,AmPmString);
  END;
END TimeToString;

PROCEDURE ValidTime(testval: Time): BOOLEAN;
  VAR
    ErrorMessage: String;
BEGIN
  ErrorMessage:="";
  WITH testval DO
    IF Hour<0 THEN ErrorMessage:="Negative hours!" END;
    IF Hour>23 THEN ErrorMessage:="Hours greater than 23!" END;
    IF Minute<0 THEN ErrorMessage:="Negative minutes!" END;
    IF Minute>59 THEN ErrorMessage:="Minutes greater than 59!" END;
    IF Second<0 THEN ErrorMessage:="Negative seconds!" END;
    IF Second>59 THEN ErrorMessage:="Seconds greater than 59!" END;
  END;
  IF ErrorMessage[0]<>0C THEN
    TimeError:=ErrorMessage;
    RETURN FALSE;
  ELSE
    RETURN TRUE;
  END;
END ValidTime;

PROCEDURE SetTime(hour,min,sec: CARDINAL; phase: TimePhase;
                  VAR timeval: Time): BOOLEAN;
  VAR
    Valid: BOOLEAN;
BEGIN                  
  IF (phase<>mil) AND (hour>12) THEN
    TimeError:="Hour greater than twelve and not military time!";
    Valid:=FALSE;
  ELSE
    WITH timeval DO
      Minute:=min;
      Second:=sec;
      IF (phase=am) AND (hour=12) THEN hour:=0 END;
      IF (phase=pm) AND (hour<>12) THEN hour:=hour+12 END;
      Hour:=hour;
    END;
    Valid:=ValidTime(timeval);
  END;
  IF NOT Valid THEN
    WITH timeval DO
      Minute:=0;
      Second:=0;
      Hour:=0;
    END;
  END;
  RETURN Valid;
END SetTime;

PROCEDURE TimeToSerialSec(tval: Time; VAR tsec: LONGINT);
BEGIN
  WITH tval DO
    tsec:=SecsPerHour*LONGCARD(Hour)+
            SecsPerMinute*LONGCARD(Minute)+LONGCARD(Second);
  END;
END TimeToSerialSec;

PROCEDURE SerialSecToTime(VAR tval: Time; tsec: LONGINT);
  VAR
    SecsLeft: LONGINT;
    HourLI,MinLI: LONGINT;
BEGIN
  SecsLeft:=tsec-SecsPerDay*(tsec DIV SecsPerDay);
  WITH tval DO
    HourLI:=SecsLeft DIV SecsPerHour;
    SecsLeft:=SecsLeft-HourLI*SecsPerHour;
    MinLI:=SecsLeft DIV SecsPerMinute;
    SecsLeft:=SecsLeft-SecsPerMinute*MinLI;
    Hour:=CARDINAL(HourLI);
    Minute:=CARDINAL(MinLI);
    Second:=CARDINAL(SecsLeft);
  END;
END SerialSecToTime;

PROCEDURE TimeFormatToString(VAR fmtstg: ARRAY OF CHAR);

BEGIN
  CopyString(fmtstg,"HH");
  ConcatString(fmtstg,TimeFormat.Separator);
  ConcatString(fmtstg,"MM");
  IF TimeFormat.Style=HHMMSS THEN
    ConcatString(fmtstg,TimeFormat.Separator);
    ConcatString(fmtstg,"SS");
  END;
  ConcatString(fmtstg," [am/pm/<blank>=mil]");
END TimeFormatToString;

PROCEDURE StringToTime(instg: ARRAY OF CHAR; VAR newtime: Time): BOOLEAN;
  VAR
    Token: ARRAY[0..9] OF CHAR;
    ConvResult: BOOLEAN;
    NumArgs,LastReqArg,MaxArg: CARDINAL;
    NewHr,NewMin,NewSec: CARDINAL;
    CurArg: LONGCARD;
    BrkLoop,InputOK: BOOLEAN;
    TPhase: TimePhase;
    Sep: ARRAY[0..3] OF CHAR;
BEGIN
  CopyString(Sep, TimeFormat.Separator);
  ConcatString(Sep," ");
  IF NOT InitParse(Sep) THEN
    TimeError:="StringToTime:  InitParse failed!!";
    InputOK:=FALSE;
  ELSE
    IF TimeFormat.Style=HHMMSS THEN
      LastReqArg:=2;
      MaxArg:=3;
    ELSE
      LastReqArg:=1;
      MaxArg:=2;
    END;
    TPhase:=mil;
    NumArgs:=0;
    BrkLoop:=FALSE;
    InputOK:=TRUE;
    WHILE (NOT BrkLoop) AND (InputOK) DO
      IF NOT ParseString(instg,Token) THEN
        InputOK:=FALSE;
        BrkLoop:=TRUE;
        TimeError:="StringToTime:  Parsing of input string failed!!";
      ELSE
        IF NumArgs<MaxArg THEN 
          IF ConvStringToNumber(Token,CurArg,TRUE,10) THEN
            CASE NumArgs OF
              0: NewHr  := CARDINAL(CurArg);|
              1: NewMin := CARDINAL(CurArg);|
              2: NewSec := CARDINAL(CurArg);
            END;
          ELSE
            TimeError:="StringToTime:  character data in a numeric field!!";
          END;
        ELSE
          CASE Token[0] OF
            'a': TPhase:=am;|
            'p': TPhase:=pm;|
          ELSE
            TPhase:=mil;
          END;
        END;
        IF StringLength(instg)=0 THEN
          IF NumArgs<LastReqArg THEN 
            InputOK:=FALSE;
            TimeError:="StringToTime:  not enough fields specified!!";
          END;
          BrkLoop:=TRUE;
        ELSIF NumArgs=MaxArg THEN BrkLoop:=TRUE END;
      END;
      IF NOT BrkLoop THEN INC(NumArgs) END;
    END;
    IF InputOK THEN 
      IF TimeFormat.Style=HHMM THEN NewSec:=0 END;
      IF NOT SetTime(NewHr,NewMin,NewSec,TPhase,newtime) THEN
        InputOK:=FALSE;
        TimeError:="StringToTime:  Illegal hour, minute or seconds values";
      END;
    END;
    RETURN InputOK;
  END;
END StringToTime;

BEGIN
  SetTimeFormat(HHMMSS,mil,colon);
END Time.
